/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                          Coherent mode SDK DEMO                           */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#ifndef __U_DEVICE_PANEL_H__
#define __U_DEVICE_PANEL_H__

#include <wx/wx.h>
#include <wx/sizer.h>
#include <wx/tglbtn.h>
#include <wx/combobox.h>
#include "g65ddcapi.h"
#include "u_spectrum.h"
#include "u_fft.h"
#include "u_floatedit.h"

class TMainWindow;

class TDevicePanel:public wxPanel
{
public:
    void DDC1StreamCallback(const void *Buffer,uint32_t Count,uint32_t BitsPerSample);
    void ADCSnapshotCallback(const int16_t *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t MaxADCAmplitude);

    void UpdateControls(void);
    void UpdateDDC1(void);
    void UpdateDDC1Frequency(void);
    void UpdateRange(void);
    void UpdateDeviceState(void);

    TDevicePanel(TMainWindow *MainWindow,ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex);
    ~TDevicePanel(void);
protected:
private:
    ICohG65DDCDeviceSet     *m_DeviceSet;
    G65DDC_DEVICE_INFO      m_DeviceInfo;
    uint32_t                m_DeviceIndex;
    TMainWindow             *m_MainWindow;
    wxStaticText            *m_LSerNum;
    wxStaticText            *m_LPhaseShift;
    TFloatEdit              *m_EPhaseShift;
    wxStaticText            *m_LState;
    TSpectrum               *m_DDC1Spectrum;
    TFFTBuffer              m_DDC1FFTOverlappingBuffer;
    float                   m_DDC1FFTCoeffs[FFT_SIZE];
    float                   m_DDC1FFTCompensation[FFT_SIZE];
    float                   m_DDC1FFTBuffer[2*FFT_SIZE];
    wxMutex                 m_DDC1FFTMutex;
    G65DDC_DDC_INFO         m_DDC1Info;

    TSpectrum               *m_ADCSpectrum;
    float                   m_ADCFFTBuffer[2*FFT_SIZE];
    float                   m_ADCFFTCoeffs[FFT_SIZE];
    float                   m_ADCFFTCompensation[FFT_SIZE/2];
    double                  m_ADCSpectrumOffset;
    wxMutex                 m_ADCFFTMutex;
    uint32_t                m_ADCSpectrumBW;
    uint32_t                m_ADCSpectrumCenterFrequency;
    uint32_t                m_RangeCenterFrequency;


    void UpdateDDC1FFTCompensation(void);

    void OnSpectrumTune(TSpectrumTuneEvent &Event);
    void OnNumericEditChanged(wxNotifyEvent &Event);

    DECLARE_EVENT_TABLE()
};

#endif

