/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                          Coherent mode SDK DEMO                           */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#ifndef __U_MAINWINDOW_H__
#define __U_MAINWINDOW_H__

#include <wx/wx.h>
#include <wx/sizer.h>
#include <wx/tglbtn.h>
#include <wx/combobox.h>
#include "g65ddcapi.h"
#include "u_spectrum.h"
#include "u_frequencyedit.h"
#include "u_devicepanel.h"
#include "u_phase.h"

class TMainWindow:public wxFrame,public ICohG65DDCDeviceSetCallback
{
public:
    void SetDDC1Frequency(uint32_t Frequency);

    TMainWindow(ICohG65DDCDeviceSet *DeviceSet);
    ~TMainWindow(void);
private:
    ICohG65DDCDeviceSet         *m_DeviceSet;
    uint32_t                    m_DeviceCount;
    G65DDC_DEVICE_INFO          *m_DeviceInfos;

    wxStaticText                *m_LDDC1Frequency;
    TFrequencyEdit              *m_EDDC1Frequency;
    wxStaticText                *m_LDDC1BW;
    wxComboBox                  *m_CBDDC1BW;

    wxStaticText                *m_LDDC1Counter;
    wxStaticText                *m_LDDC1CounterValue;

    wxStaticText                *m_LADCCounter;
    wxStaticText                *m_LADCCounterValue;

    wxStaticText                *m_LDDC1SampleRate;
    wxStaticText                *m_LDDC1SampleRateValue;

    wxStaticText                *m_LADCSampleRate;
    wxStaticText                *m_LADCSampleRateValue;

    TDevicePanel                **m_DevicePanels;

    wxToggleButton              *m_BPower;
    wxToggleButton              *m_B1PPS;

    G65DDC_DDC_INFO             m_DDC1Info;

    wxTimer                     m_InitPowerOnTimer;
    wxTimer                     m_TimestampTimer;
    wxTimer                     m_DeviceStateTimer;

    uint64_t                    m_PrevADCCounter;
    double                      m_PrevDDC1Counter;

    uint32_t                    m_DevicePairCount;
    TPhase                      **m_Phases;

    uint64_t                    m_DDC1SampleCounter;

    //callbacks
    virtual void CohG65DDC_ADCSnapshotCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel);
    virtual void CohG65DDC_DDC1StreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceCount,const void **Buffers,uint32_t Count,uint32_t BitsPerSample);
    virtual void CohG65DDC_DDC2StreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t Count);
    virtual void CohG65DDC_DDC2PreprocessedStreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t Count,float Peak,float RMS);
    virtual void CohG65DDC_AudioStreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,uint32_t Stage,const float *Buffer,uint32_t Count);

    void SetPower(bool Power);

    void OnButtonClick(wxCommandEvent &Event);
    void OnComboBoxChanged(wxCommandEvent &Event);
    void OnNumericEditChanged(wxNotifyEvent &Event);
    void OnTimer(wxTimerEvent& Eevnt);

    void GetPhase_int16(int16_t *Buffer1,int16_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To);
    void GetPhase_int32(int32_t *Buffer1,int32_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To);

    void Update1PPSControls(void);

    DECLARE_EVENT_TABLE()
};

#endif

