//This example ilustrates how to use object-oriented interface of the G65DDC API
//in non-object C language. It is recomended to use object-oriented interface
//instead of non-object interface in C because it is more efecient than non-object
//interface. Each function of non-object interface verifies device handle
//and translates the handle to reference to internal data structures, methods
//of object interface do not do it.

#include "g65ddcapi.h"
#include <stdio.h>
#include <math.h>
#include <dlfcn.h>
#include <inttypes.h>
#include <pthread.h>
#include <errno.h>
#include <inttypes.h>
#include <pulse/simple.h>

#define AUDIO_SAMPLE_RATE 48000
                                       
//prototypes of callback methods
void MyCallback_G65DDC_DDC1StreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC1ChannelId,const void *Buffer,uint32_t Count,uint32_t BitsPerSample);
int MyCallback_G65DDC_DDC1PlaybackStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC1ChannelId,void *Buffer,uint32_t Count,uint32_t BitsPerSample);
void MyCallback_G65DDC_DDC2StreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count);
void MyCallback_G65DDC_DDC2PreprocessedStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,float SlevelPeak,float SlevelRMS);
void MyCallback_G65DDC_ADCSnapshotCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,const int16_t *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel);
void MyCallback_G65DDC_AudioStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,uint32_t Stage,const float *Buffer,uint32_t Count);
int MyCallback_G65DDC_AudioPlaybackStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,float *Buffer,uint32_t Count);

IG65DDCDevice *Device;
void *API=NULL;
G65DDC_CREATE_INSTANCE CreateInstance;
pa_simple *PaSimple=NULL;

//table of virtual methods
IG65DDCDeviceCallbackVtbl MyCallbackMethodsTable=
{    
    .G65DDC_DDC1StreamCallback=&MyCallback_G65DDC_DDC1StreamCallback,
    .G65DDC_DDC1PlaybackStreamCallback=&MyCallback_G65DDC_DDC1PlaybackStreamCallback,
    .G65DDC_DDC2StreamCallback=&MyCallback_G65DDC_DDC2StreamCallback,
    .G65DDC_DDC2PreprocessedStreamCallback=&MyCallback_G65DDC_DDC2PreprocessedStreamCallback,
    .G65DDC_ADCSnapshotCallback=&MyCallback_G65DDC_ADCSnapshotCallback,    
    .G65DDC_AudioStreamCallback=&MyCallback_G65DDC_AudioStreamCallback,
    .G65DDC_AudioPlaybackStreamCallback=&MyCallback_G65DDC_AudioPlaybackStreamCallback
};

//callback object
IG65DDCDeviceCallback MyCallbackObject=
{
    &MyCallbackMethodsTable
};


void MyCallback_G65DDC_DDC1StreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC1ChannelId,const void *Buffer,uint32_t Count,uint32_t BitsPerSample)
{
    //it does nothing in this example
}

int MyCallback_G65DDC_DDC1PlaybackStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC1ChannelId,void *Buffer,uint32_t Count,uint32_t BitsPerSample)
{
    //it does nothing in this example
    return 0;
}

void MyCallback_G65DDC_DDC2StreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count)
{
    //it does nothing in this example
}

void MyCallback_G65DDC_DDC2PreprocessedStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,float SlevelPeak,float SlevelRMS)
{
 double Slevel_dBm;

    //convert slevel RMS in Volts to dBm
    Slevel_dBm=10.0*log10(SlevelRMS*SlevelRMS*(1000.0/50.0));

    printf("\rSlevel: RMS[V]=%9.7f V, RMS[dBm]=%6.1f dBm, Peak[V]=%9.7f V",SlevelRMS,Slevel_dBm,SlevelPeak);
    fflush(stdout);
}

void MyCallback_G65DDC_ADCSnapshotCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,const int16_t *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel)
{
    //it does nothing in this example
}

void MyCallback_G65DDC_AudioStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,uint32_t Stage,const float *Buffer,uint32_t Count)
{ 
 int Error;
 
    if(Stage==2 && PaSimple!=NULL)
    {
        //we want to play only audio buffers of type 2 which means final audio samples (audio gain, audio filter and volume are applied)
        pa_simple_write(PaSimple,Buffer,Count*2*sizeof(float),&Error);
    }  
}

int MyCallback_G65DDC_AudioPlaybackStreamCallback(IG65DDCDeviceCallback *Callback,IG65DDCDevice *Device,uint32_t DDC2ChannelId,float *Buffer,uint32_t Count)
{
    //it does nothing in this example
    return 0;
}



int  Initialize(void)
{ 
 pa_sample_spec SampleFormat;
 int Error;
 
    API=dlopen("libg65ddcapi.so",RTLD_LAZY);

    if(API==NULL)
    {
        printf("Failed to load libg65ddcapi.so. %s\n",dlerror());
        return 0;
    }

    //retrieve addresse of CreateInstance API functions
    CreateInstance=(G65DDC_CREATE_INSTANCE)dlsym(API,"CreateInstance");

    //create instance of device object
    if(!CreateInstance(G65DDC_CLASS_ID_DEVICE,(void**)&Device))
    {
        printf("Failed to create device object. Error code=%u\n",errno);
        dlclose(API);
        return 0;
    }

    //open the first available G65DDC device
    if(!Device->lpVtbl->Open(Device,G65DDC_OPEN_FIRST))
    {
        printf("No available G65DDC device found.\n");
        Device->lpVtbl->Free(Device);
        dlclose(API);
        return 0;
    }
              
    //initialize pulse audio
    SampleFormat.channels=2;
    SampleFormat.rate=AUDIO_SAMPLE_RATE;
    SampleFormat.format=PA_SAMPLE_FLOAT32LE;
    
    PaSimple=pa_simple_new(NULL,"G65DDC SDK EXAMPLE",PA_STREAM_PLAYBACK,NULL,"Audio0",&SampleFormat,NULL,NULL,&Error);
    
    if(PaSimple==NULL)
    {
        printf("Failed to initialize pulse audio.\n");
    }
    
    return 1;
}


int main(void)
{
 G65DDC_DDC_INFO DDCInfo;
 uint32_t DDCTypeIndex; 
 int32_t Frequency;
 int32_t DDC1Frequency;
 int32_t DDC2Frequency;
 int32_t DemodulatorFrequency;
 uint32_t SamplesPerBuffer;
 uint32_t DDC1Id;
 uint32_t DDC2Id;

    if(!Initialize())
    {
        return -1;
    }

    //set power
    Device->lpVtbl->SetPower(Device,1);

    //create DDC1 channel
    Device->lpVtbl->CreateDDC1(Device,&DDC1Id);
    
    //create DDC2 channel
    Device->lpVtbl->CreateDDC2(Device,DDC1Id,&DDC2Id);
    
    //set DDC type of DDC1 to 1 (DDC Bandwidth=24000, SampleRate=32kHz, BitsPerSample=32)
    Device->lpVtbl->SetDDC(Device,DDC1Id,1);

    //retrieve current DDC type of the DDC1
    Device->lpVtbl->GetDDC(Device,DDC1Id,&DDCTypeIndex,&DDCInfo);
    printf("DDC type of DDC1: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);

    
    //set DDC type of DDC2 to 1 (DDC Bandwidth=24000, SampleRate=32kHz, BitsPerSample=32)
    Device->lpVtbl->SetDDC(Device,DDC2Id,1);

    //retrieve current DDC type of the DDC2
    Device->lpVtbl->GetDDC(Device,DDC2Id,&DDCTypeIndex,&DDCInfo);    
    printf("DDC type of DDC2: Index=%u, Bandwidth=%u, SampleRate=%u, BitsPerSample=%u\n",DDCTypeIndex,DDCInfo.Bandwidth,DDCInfo.SampleRate,DDCInfo.BitsPerSample);



    //register callback object
    Device->lpVtbl->SetCallback(Device,&MyCallbackObject);

    //tune to an AM station frequency
    Device->lpVtbl->SetFrequency(Device,DDC2Id,702000);
    //it is possible to tune DDC1, DDC2 and demodulator directly using
    //SetDDCFrequency and SetDemodulatorFrequency
    //Device->lpVtbl->SetFrequency(Device,DDC2Id,702000) can be replaced by the following:
    //Device->lpVtbl->SetRange(Device,G65DDC_RANGE_1);
    //Device->lpVtbl->SetDDCFrequency(Device,DDC1Id,700000);
    //Device->lpVtbl->SetDDCFrequency(Device,DDC2Id,0);
    //Device->lpVtbl->SetDemodulatorFrequency(Device,DDC2Id,2000);
    //final absolute frequency of the demodulator is 700000+0+2000=702000

    //get absolute frequency where the demodulator is tuned
    Device->lpVtbl->GetFrequency(Device,DDC2Id,&Frequency);

    //get DDC1 frequency
    Device->lpVtbl->GetDDCFrequency(Device,DDC1Id,&DDC1Frequency);

    //get relative DDC2 frequency
    Device->lpVtbl->GetDDCFrequency(Device,DDC2Id,&DDC2Frequency);

    //get relative demodulator frequency
    Device->lpVtbl->GetDemodulatorFrequency(Device,DDC2Id,&DemodulatorFrequency);

    printf("Listening to (demodulator absolute frequency): %d Hz\n",Frequency);
    printf("DDC1 frequency: %d Hz\n",DDC1Frequency);
    printf("DDC2 frequency (relative): %d Hz\n",DDC2Frequency);
    printf("Demodulator frequency (relative): %d Hz\n",DemodulatorFrequency);


    //set AGC params, attack=0.02s, decay=0.5s, ref. level=-15dB
    Device->lpVtbl->SetAGCParams(Device,DDC2Id,0.020,0.500,-15.0);

    //enable AGC
    Device->lpVtbl->SetAGC(Device,DDC2Id,1);

    //set AM mode
    Device->lpVtbl->SetDemodulatorMode(Device,DDC2Id,G65DDC_MODE_AM);

    //set demodulator filter bandwidth for to 8kHz
    Device->lpVtbl->SetDemodulatorFilterBandwidth(Device,DDC2Id,8000);
    
    //calculate number of samples per DDC1 buffer - 100 ms
    Device->lpVtbl->GetDDC(Device,DDC1Id,&DDCTypeIndex,&DDCInfo);
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);

    //start DDC1 streaming which has to be running before DDC2 is started
    Device->lpVtbl->StartDDC(Device,DDC1Id,SamplesPerBuffer);

    printf("\nPress ENTER to exit.\n\n");

    //calculate number of samples per DDC2 buffer - 100 ms
    Device->lpVtbl->GetDDC(Device,DDC2Id,&DDCTypeIndex,&DDCInfo);    
    SamplesPerBuffer=100*DDCInfo.SampleRate/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start DDC2 streaming, it has to be running before StartAudio is called
    Device->lpVtbl->StartDDC(Device,DDC2Id,SamplesPerBuffer);

    //calculate number of samples per audio buffer - 100 ms    
    SamplesPerBuffer=100*AUDIO_SAMPLE_RATE/1000;
    //round to 64 samples
    SamplesPerBuffer=(SamplesPerBuffer+63)&(~63);
    
    //start audio streaming for channel 0
    Device->lpVtbl->StartAudio(Device,DDC2Id,SamplesPerBuffer);

    getchar();

    //stop audio streaming
    Device->lpVtbl->StopAudio(Device,DDC2Id);

    //stop DDC2 streaming
    Device->lpVtbl->StopDDC(Device,DDC2Id);

    //stop DDC1 streaming
    Device->lpVtbl->StopDDC(Device,DDC2Id);

    
    //unregister callback object
    Device->lpVtbl->SetCallback(Device,NULL);

    Device->lpVtbl->SetPower(Device,0);

    Device->lpVtbl->Free(Device);
    
    if(PaSimple)
    {
        pa_simple_free(PaSimple);
    }

    dlclose(API);

    return 0;
}